@@ -41,7 +41,7 @@ import PEATDB.Ekin.Fitting as Fitting
 class Pipeline(object):
     """This class does all the pipeline processing and configuration"""
 
-    __version__ = '1.1.0'
+    __version__ = '1.2.0'
     configsections = ['base','files','fitting','models','variables','functions',
                       'excel','plotting','custom']
 
@@ -60,7 +60,6 @@ class Pipeline(object):
         self.queue = {}
         self.results = []
         self.sep = '__'   #symbol for internal separator
-
         return
 
     def createConfig(self, conffile='default.conf', **kwargs):
@@ -72,17 +71,19 @@ class Pipeline(object):
         defaults = {'base': [('format', 'databyrow'), ('rowstart', 0), ('colstart', 0), ('rowend', 0),
                         ('colend', 0), ('colheaderstart', 0),('rowheaderstart', 0),
                         ('rowheader', ''), ('colheader', ''),
+                        ('colheaderlabels', ''), ('rowheaderlabels', ''),
                         ('rowrepeat', 0), ('colrepeat', 0), ('delimeter', ','),
                         ('workingdir', wdir), ('functionsconf',functionsconf),
                         ('ignorecomments', 1),
                         ('decimalsymbol', '.'), ('preprocess',''),
-                        ('xformat',''),('yformat','')],
-                    'files': [('groupbyname', 0), ('parsenamesindex', 0),
-                                ('replicates',0)],
+                        ('xformat',''), ('yformat',''), ('groupbyfields', 0)],
+                    'files': [('groupbyname', 0), ('parsenamesindex', ''),
+                              ('filenameseparator',''), ('parsemethod','words'),
+                              ('replicates',0), ('extension','.txt')],
                     'models': [('model1', '')], 'variables': [('variable1', '')],
                     'functions':[('function1','')],
                     'excel': [('sheet', 0), ('numsheets', 1)],
-                    'plotting': [('saveplots', 0), ('fontsize',9),                                    
+                    'plotting': [('saveplots', 0), ('fontsize',9),
                             ('normalise', 0), ('grayscale', 0), ('alpha',0.8),
                             ('font','sans-serif'), ('markersize',25), ('linewidth',1),
                             ('showerrorbars',0), ('dpi', 100), ('marker','o'),
@@ -170,18 +171,21 @@ class Pipeline(object):
         return
 
     def loadModels(self):
-        print self.modelsfile
+        #print self.modelsfile
         if self.modelsfile != '':
             try:
                 Fitting.loadModelsFile(self.modelsfile)
             except:
                 pass
-        print Fitting.currentmodels.keys()
+        #print Fitting.currentmodels.keys()
         return
 
     def openRaw(self, filename=None, callback=None):
         """Open raw file, display preview and get some info about them"""
 
+        if not os.path.exists(filename):
+            return None
+
         if os.path.splitext(filename)[1] == '.xls':
             lines = self.openExcel(filename)
         else:
@@ -278,7 +282,7 @@ class Pipeline(object):
 
     def doProcessingStep(self, data, fname):
         """Apply a pre-defined processing step to the data"""
-       
+
         X = Processor(self.functionsconf)
         names = [i[1] for i in self.functions]
         for n in names:
@@ -287,15 +291,29 @@ class Pipeline(object):
                 return data
         #user may want to see raw data before processing, so we plot here
         if self.saveplots == 1:
-            Er = self.getEkinProject(data)            
+            Er = Utilities.getEkinProject(data)
         data = X.doFunctions(names, data)
         if self.saveplots == 1:
-            Em = self.getEkinProject(data)      
+            Em = Utilities.getEkinProject(data)
             Er.addProject(Em,label='proc')
             Er.saveProject(fname+'_processed')
             self.saveEkinPlotstoImages(Er, fname+'_processed',groupbylabel=True)
         return data
 
+    def parseLabels(self):
+        """Get labels from filenames"""
+
+        #parseindex can be a list, we currently just take first element
+        if type(self.parsenamesindex) is not types.IntType and self.parsenamesindex!='':
+            index = self.parsenamesindex.split(',')
+            index = int(index[0])
+        else:
+            index = self.parsenamesindex
+        self.namelabels = Utilities.parseFileNames(self.queue, ind=index,
+                                                     sep=self.filenameseparator,
+                                                     match=self.parsemethod)
+        return
+
     def run(self, callback=None):
         """Do initial import/fitting run with the current config"""
 
@@ -304,30 +322,42 @@ class Pipeline(object):
         self.prepareData()
         print 'processing files in queue..'
 
-        if self.groupbyname == 1:
-            labels = self.parseFileNames(self.queue, ind=self.parsenamesindex)
-        else:
-            labels = self.queue
-
-        self.labels = labels
+        self.parseLabels()
         imported = {}   #raw data
         results = {}    #fitted data
-
-        #print 'queue'
         #print self.queue
-        #print filelabels
 
         for key in self.queue:
             filename = self.queue[key]
             lines = self.openRaw(filename)
+            if lines == None:
+                continue
             data = self.doImport(lines)
             imported[key] = data
 
+        #rebuild dict into a nested structure if it's flat (i.e. from single files)
+        '''from Data import NestedData
+        D = NestedData(imported)
+        D.buildNestedStructure([0,2])
+        D.show()
+        imported = D.data
+        self.namelabels = None'''
+
         #try to average replicates here before we process
         if self.replicates == 1:
-            imported = self.handleReplicates(imported)
+            if self.namelabels != None:
+                imported = Utilities.addReplicates(imported, self.namelabels)
+            else:
+                print 'no replicates detected from labels'
+
+        #re-arrange the imported dict if we want to group our output per field
+        if self.groupbyfields == 1:
+            imported = Utilities.arrangeDictbySecondaryKey(imported, self.namelabels)
 
         total = len(imported)
+        #print imported
+        #print self.namelabels
+
         c=0.0
         for key in imported:
             if self.stop == True:
@@ -341,15 +371,17 @@ class Pipeline(object):
             if self.function1 != '':
                 data = self.doProcessingStep(data, fname)
 
-            if labels.has_key(key):
-                label = labels[key]
+            if self.namelabels == None or not self.namelabels.has_key(key):
+                namelabel = key
             else:
-                label = key
-            #print key,fname,label
+                namelabel = self.namelabels[key]
+            #print namelabel, key
+            #print data
 
             #if we have models to fit this means we might need to propagate fit data
             if self.model1 != '':
                 Em = EkinProject()
+                #grouping by file labels handled here
                 if self.groupbyname == 1:
                     #we don't pass the last model if it has to be
                     #reserved for a final round of fitting from the files dict
@@ -359,16 +391,17 @@ class Pipeline(object):
                                                models=models,variables=variables)
                 else:
                     E,fits = self.processFits(rawdata=data, Em=Em)
-                results[label] = fits
+                results[namelabel] = fits
+                #print E.datasets, namelabel
             else:
-                 #if no fitting we just put the data in ekin
-                Em = self.getEkinProject(data)
-                results = data
+                #if no fitting we just put the data in ekin
+                Em = Utilities.getEkinProject(data)
+                results[namelabel] = data
+
             Em.saveProject(fname)
-            Em.exportDatasets(fname)
+            Em.exportDatasets(fname, append=True)
             if self.model1 != '':
                 self.saveFitstoCSV(Em, fname)
-
             if self.saveplots == 1:
                 self.saveEkinPlotstoImages(Em, fname)
             c+=1.0
@@ -377,16 +410,15 @@ class Pipeline(object):
 
         #if grouped by file names then we process that here from results
         if self.groupbyname == 1:
-            #print results.keys()
-            results = self.extractSecondaryKeysFromDict(results)
+            results = Utilities.extractSecondaryKeysFromDict(results)
             Em = EkinProject()
+            #print results
             E,fits = self.processFits(rawdata=results, Em=Em)
             fname = os.path.join(self.workingdir, 'final')
             Em.saveProject(os.path.join(self.workingdir, fname))
             Em.exportDatasets(os.path.join(self.workingdir, fname))
             if self.model1 != '':
                 self.saveFitstoCSV(Em, fname)
-            #if self.saveplots == 1:
             self.saveEkinPlotstoImages(Em, fname)
         print 'processing done'
         print 'results saved to %s' %self.workingdir
@@ -402,6 +434,7 @@ class Pipeline(object):
             returns: final set of fits"""
 
         nesting = self.getDictNesting(rawdata)
+
         if models == None:
             models = self.models
             variables = self.variables
@@ -427,9 +460,10 @@ class Pipeline(object):
         #print nesting,currmodel,currvariable,parentkey
 
         if nesting == 0:
-            #final level of nesting, we just fit
+            #bottom level of nesting, we just fit
             xerror = float(self.xerror); yerror = float(self.yerror)
-            E = self.getEkinProject(rawdata, xerror=xerror, yerror=yerror)
+            E = Utilities.getEkinProject(rawdata, xerror=xerror, yerror=yerror)
+            print rawdata.keys(), parentkey, currmodel
             E,fit = self.getFits(E, currmodel, currvariable, str(parentkey))
             Em.addProject(E, label=parentkey)
             return E,fit
@@ -441,52 +475,15 @@ class Pipeline(object):
                     lbl = str(l)+'_'+str(parentkey)
                 else:
                     lbl = l
-                #now we pass each child node to the same function
+                #now we pass each child node recursively
                 E,fit = self.processFits(rawdata[l], ind=ind-1, parentkey=lbl, Em=Em)
                 fitdata[l] = fit
-            E = self.getEkinProject(fitdata)
+            E = Utilities.getEkinProject(fitdata)
             if parentkey == '': parentkey = 'final'
             E,fit = self.getFits(E, currmodel, currvariable, str(parentkey))
             Em.addProject(E,label=parentkey)
             return E,fit
 
-    def handleReplicates(self, data):
-        """If the configuration specifies replicates than we average the
-           corresponding data points in the raw dict """
-
-        print 'processing replicates..'
-        import operator
-        from itertools import groupby
-        newdata = {}
-        labels = self.labels
-        sorteditems = sorted(labels.iteritems(), key=operator.itemgetter(1))
-
-        for key, group in groupby(sorteditems, lambda x: x[1]):
-            c = 0
-            subdata = []
-            for g in group:
-                f = g[0]    #key in corresponding data dict
-                c+=1
-                subdata.append(data[f])
-                print f
-            newdata[key] = self.averageDicts(subdata)
-            print '%s replicates for label %s' %(c, key)
-        #print newdata
-        return newdata
-
-    def averageDicts(self, dictslist):
-        """Average dicts of the form
-           {label1: [[x1],[y1]],..],label2:[[x2],[y2]]...}"""
-
-        newdata = {}
-        names = dictslist[0].keys()
-        for n in names:
-            arrs = []
-            for D in dictslist:
-                arrs.append(np.array(D[n]))
-            newdata[n] = list(sum(arrs)/len(arrs))
-        return newdata
-
     def getFits(self, E, model, varname='a', filename=None):
         """Fit an Ekin project
            model: model to fit
@@ -533,33 +530,18 @@ class Pipeline(object):
         return
 
     def getDictNesting(self, data):
-        """Get level of nesting"""
+        """Find if bottom level of nesting or not"""
         for d in data.keys():
             if type(data[d]) is types.DictType:
                 return 1
             else:
                 return 0
 
-    def parseFileNames(self, filenames, ind=0):
-        """Parse file names to extract a numerical value
-           ind: extract the ith instance of a number in the filename"""
-        labels = {}
-        for f in filenames:
-            bname = os.path.basename(f)
-            l = re.findall("([0-9.]*[0-9]+)", bname)[ind]
-            labels[f] = l
-            print ind, f, labels[f]
-        return labels
-
-    def parseString(self, filename):
-        """Parse string to extract a pattern"""
-
-        return
-
-    def addFolder(self, path, ext='.txt'):
+    def addFolder(self, path):
         """Add a folder to the queue"""
 
         D = {}
+        ext = self.extension
         #print os.path.basename(path), os.path.normpath(path)
         for root, dirs, files in os.walk(path):
             for d in dirs:
@@ -567,14 +549,12 @@ class Pipeline(object):
                     dirs.remove(d)
             for f in files:
                 fname = os.path.join(root,f)
-                #absname = os.path.abspath(fname)
-                if ext in os.path.splitext(fname)[1]:
+                if ext in os.path.splitext(fname)[1] or ext == '':
                     #print root,fname
                     label = os.path.relpath(fname,  os.path.normpath(path))
                     label = label.replace(os.path.sep,'_')
                     #print label
                     D[label] = fname
-        #self.addtoQueue(D.values())
         self.queue = D
         return D
 
@@ -594,7 +574,7 @@ class Pipeline(object):
         #print self.plotting
         for k in opts:
             if k=='markers':
-                opts[k]=list(opts[k].split(','))                
+                opts[k]=list(opts[k].split(','))
             else:
                 try:
                     opts[k] = eval(opts[k])
@@ -604,17 +584,17 @@ class Pipeline(object):
 
     def saveEkinPlotstoImages(self, E, filename, groupbylabel=False):
         """Save the ekin plots to png images
-           groupbylabel allows commonly labelled datasets 
+           groupbylabel allows commonly labelled datasets
            to be overlayed - useful for viewing processing results"""
         title = os.path.basename(filename)
         filename = os.path.join(self.workingdir, filename)
         filename = filename + '.png'
         #limit to 30 plots per image
         dsets = E.datasets
-        if len(dsets) > 30: 
+        if len(dsets) > 30:
             dsets = E.datasets[:30]
         plotopts = self.getPlotOptions()
-        #the following functionality should be integrated into ekin plotter    
+        #the following functionality should be integrated into ekin plotter
         if groupbylabel == True:
             import itertools
             import pylab as plt
@@ -629,7 +609,7 @@ class Pipeline(object):
             cols, dim = E.getGeometry(groups, 0)
             c=1
             for g in groups:
-                ax=fig.add_subplot(cols,dim,c)                            
+                ax=fig.add_subplot(cols,dim,c)
                 E.plotDatasets(g,figure=fig,axis=ax,plotoption=3,
                                 varyshapes=1,**plotopts)
                 c+=1
@@ -650,80 +630,3 @@ class Pipeline(object):
         EM.exportCSV(filename)
         return
 
-    @classmethod
-    def getEkinProject(self, data, xerror=None, yerror=None, sep='__'):
-        """Get an ekin project from a dict of the form
-             {label:([x],[y]),..} or
-             {label:([x],[y],[xerr],[yerr]),..}"""
-
-        E = EkinProject(mode='General')
-        for d in data.keys():
-            if type(data[d]) is types.DictType:
-                for lbl in data[d]:
-                    name = str(d)+sep+str(lbl)
-                    xy = data[d][lbl]
-                    ek=EkinDataset(xy=xy)
-                    E.insertDataset(ek, name)
-            else:
-                #print data[d]
-                if len(data[d]) == 4:
-                    x,y,xerrs,yerrs = data[d]
-                else:
-                    x,y = data[d]
-                    xerrs = []; yerrs=[]
-                    if xerror!=None:
-                        xerrs=[xerror for i in x]
-                    if yerror!=None:
-                        yerrs=[yerror for i in y]
-                ek = EkinDataset(xy=[x,y], xerrs=xerrs, yerrs=yerrs)
-                E.insertDataset(ek, d)
-                #print ek.errors
-        return E
-
-    @classmethod
-    def extractSecondaryKeysFromDict(self, data):
-        """Re-arrange a dict of the form {key1:([names],[values]),..
-           into the form {name1:([keys],[values]),..}"""
-
-        newdata = {}
-        kys = data.keys()
-        datasets = data[kys[0]][0]
-        for d in datasets:
-            newdata[d] = []
-        for l in data:
-            names, vals, xerr, yerr = data[l]
-            #print zip(names, vals)
-            for d,v in zip(names, vals):
-                newdata[d].append((l,v))
-        for d in newdata:
-            newdata[d] = zip(*newdata[d])
-        return newdata
-
-class FolderStructure(object):
-    """Class to hold structure rawdata that makes handling it easier"""
-
-    def __init__(self, path):
-        self.path = path
-        self.rootdir = os.path.basename(path)
-        self.info = Utilities.getdirectoryStructure(path)
-        print self.rootdir
-        print '------------------'
-        print self.info
-        self.traverse(self.info)
-        return
-
-    def traverse(self, data):
-        for i in data:
-            if type(data[i]) is types.DictType:
-                self.traverse(data[i])
-            else:
-                print i, data[i]
-
-    def report(self):
-        """report on structure found in path"""
-        print 'rootdir is %s' %self.rootdir
-        folders = len(self.info)
-        print '%s files found in %s folders' %(self.p)
-        return
-
-
