@@ -42,6 +42,7 @@ from Helper import HelperDialog
 from Editor import TextEditor
 from Rename import BatchRenameApp
 import Images
+import Utilities
 
 class PipeApp(Frame, GUI_help):
     """Data pipe GUI for importing and fitting of raw data.
@@ -195,7 +196,8 @@ class PipeApp(Frame, GUI_help):
 
     def createConfig(self):
         filename = self.saveFilename()
-        self.p.createConfig(filename)
+        if filename:
+            self.p.createConfig(filename)
         return
 
     def loadConfig(self, filename=None):
@@ -209,7 +211,7 @@ class PipeApp(Frame, GUI_help):
     def reloadConfig(self):
         self.loadConfig(self.p.configurationfile)
         return
-        
+
     def editConfig(self):
         self.editFile(self.p.configurationfile)
         return
@@ -305,9 +307,8 @@ class PipeApp(Frame, GUI_help):
 
     def runTests(self):
         """Run tests"""
-        from Testing import Tester
-        t=Tester()
-        t.formatTests(t.basictests)
+        import Testing
+        Testing.formatTests(Testing.basictests)
         print 'tests completed ok'
         return
 
@@ -396,7 +397,7 @@ class PipeApp(Frame, GUI_help):
         return
 
     def startTextEditor(self):
-        t = TextEditor(parent=self)        
+        t = TextEditor(parent=self)
         return
 
     def batchFileRename(self):
@@ -431,7 +432,7 @@ class PipeApp(Frame, GUI_help):
         label = Label(win,image=logo)
         label.image = logo
         label.pack(fill=BOTH,padx=4,pady=4)
-        text="""DataPipeline App, Version 1.0
+        text="""DataPipeline App, Version 1.2
              is a python desktop and web application
              that uses a configuration file to automate the import of
              raw data in a variety of formats.\n
@@ -482,6 +483,9 @@ class PlotPreviewer(Frame):
         self.numplotscounter.pack()
         self.overlayvar = BooleanVar(); self.overlayvar.set(False)
         Checkbutton(fr, text='overlay plots', variable=self.overlayvar, command=self.replot).pack(side=TOP,fill=BOTH)
+        self.normalizevar = BooleanVar(); self.normalizevar.set(False)
+        Checkbutton(fr, text='normalize', variable=self.normalizevar, command=self.replot).pack(side=TOP,fill=BOTH)
+
         fr.pack(side=LEFT)
         self.plotframe = PlotPanel(parent=self, side=BOTTOM, height=200, tools=True)
         self.dsindex = 0
@@ -493,6 +497,12 @@ class PlotPreviewer(Frame):
 
     def replot(self):
         """Replot"""
+
+        if len(self.E.datasets) == 0:
+            print 'no datasets to plot, check your config'
+            self.clear()
+            return
+
         p = int(self.numplotscounter.getvalue())
         if p > self.E.length: p = self.E.length
         if p>1:
@@ -513,13 +523,13 @@ class PlotPreviewer(Frame):
         else:
             plotopt = 2
             self.plotframe.Opts.opts['title']=None
-        self.plotframe.plotCurrent(dsets,
-                                    cols=c, plotoption=plotopt)
+        self.plotframe.Opts.opts['normalise'] = self.normalizevar.get()
+        self.plotframe.plotCurrent(dsets, cols=c, plotoption=plotopt)
         return
 
     def loadData(self, data):
         """Load dict into datasets"""
-        E = self.E = Pipeline.getEkinProject(data)
+        E = self.E = Utilities.getEkinProject(data)
         self.plotframe.setProject(E)
         self.totalvar.set(self.E.length)
         return
@@ -602,13 +612,13 @@ class queueManager(Frame):
     def removeSelected(self):
         """Remove selected items from queue"""
         p = self.app.p
-        if len(self.p.queue) == 0:
+        if len(p.queue) == 0:
             return
         p.queue.values()
         selected = self.listbox.getcurselection()
         for key, value in p.queue.items():
             if value in selected:
-                del self.p.queue[key]
+                del p.queue[key]
         self.update()
         self.app.updateFromQueue()
         return
